@@ -5,6 +5,7 @@
 #' @description Minimal console output for `bgmCompare` fit objects.
 #' @param x An object of class `bgmCompare`.
 #' @param ... Ignored.
+#' @return Invisibly returns `x`.
 #' @export
 print.bgmCompare = function(x, ...) {
   arguments = extract_arguments(x)
@@ -23,6 +24,9 @@ print.bgmCompare = function(x, ...) {
 
   # Dataset info
   cat(paste0(" Number of variables: ", arguments$num_variables, "\n"))
+  if(isTRUE(arguments$standardize)) {
+    cat(" Prior standardization: enabled\n")
+  }
   if(!is.null(arguments$num_groups)) {
     cat(paste0(" Number of groups: ", arguments$num_groups, "\n"))
   }
@@ -46,6 +50,7 @@ print.bgmCompare = function(x, ...) {
 
   cat("Use the `summary()` function for posterior summaries and diagnostics.\n")
   cat("See the `easybgm` package for additional summaries and plotting.\n")
+  invisible(x)
 }
 
 
