@@ -13,6 +13,7 @@ check_non_negative_integer = function(value, name) {
 
 # Helper function for validating logical inputs
 check_logical = function(value, name) {
+
   value = as.logical(value)
   if(is.na(value)) {
     stop(sprintf("Parameter `%s` must be TRUE or FALSE. Got: %s", name, value))
@@ -20,6 +21,18 @@ check_logical = function(value, name) {
   return(value)
 }
 
+# Helper function for validating optional seed parameter
+# Returns a valid integer seed (generates one if NULL)
+check_seed = function(seed) {
+  if(is.null(seed)) {
+    return(sample.int(.Machine$integer.max, 1L))
+  }
+  if(!is.numeric(seed) || length(seed) != 1 || is.na(seed) || seed < 0) {
+    stop("Argument 'seed' must be a single non-negative integer.")
+  }
+  as.integer(seed)
+}
+
 check_model = function(x,
                        variable_type,
                        baseline_category,
