@@ -3,7 +3,7 @@
 
 
 
-arma::vec compute_Vn_mfm_sbm(arma::uword no_variables,
+arma::vec compute_Vn_mfm_sbm(arma::uword num_variables,
                              double dirichlet_alpha,
                              arma::uword t_max,
                              double lambda);
